  
  /*
  *  FALL
  *  a game by Joel Gaehwiler
  */
  
  //Imports
  import fisica.*;
  import processing.opengl.*;
  import processing.xml.*;
  import ddf.minim.*;
  import fullscreen.*;
  
  //Fonts
  PFont FONT_BIG;
  PFont FONT_SMALL;
  
  //Images
  PImage BACKGROUND_MENU;
  PImage BACKGROUND_LEVEL;
  PImage LOGO_BIG;
  PImage LOGO_SMALL;
  
  //Stages
  static int STAGE_MENU = 1;
  static int STAGE_NAME = 2;
  static int STAGE_LOADER = 3;
  static int STAGE_LEVEL = 4;
  static int STAGE_END = 5;
  static int STAGE_FINISH = 6;
  
  //Stage
  int _stage = 1;
  
  //Level
  int _levelNumber = 1;
  int _levelMax = 8;
  Level _level;
  
  //User
  String _username = "";
  
  //Time
  int _timeLevelStart;
  int _timeLevelFinish;
  
  //Sound
  Minim minim;
  AudioSample _soundBackground;
  AudioSample _soundBall;
  
  //FullScreen
  FullScreen fs;
  
  //SetUp Cycle
  void setup()
  {
    //System
    size(1500,900,OPENGL);
    smooth();
    //frameRate(50);
    
    //FullScreen
    fs = new FullScreen(this);
    fs.enter();
    
    //Engine
    Fisica.init(this);
    minim = new Minim(this);
    
    //Load Fonts
    FONT_SMALL = loadFont("text20.vlw");
    FONT_BIG = loadFont("text50.vlw");
    
    //Load Images
    BACKGROUND_MENU = loadImage("data/images/bg_menu.jpg");
    LOGO_BIG = loadImage("data/images/logo_big.png");
    LOGO_SMALL = loadImage("data/images/logo_small.png");
    
    //Load Sound
    _soundBall = minim.loadSample("sounds/tick.wav");
  }
  
  //Draw Cycle
  void draw()
  { 
    //Setup
    noStroke();
    fill(255);
    background(0);
    
    //Draw Menu
    if(_stage == STAGE_MENU)
    {    
      //Draw Images
      image(BACKGROUND_MENU,0,0);
      image(LOGO_BIG,500,100);
      
      //Draw Informations
      drawText("Version 1.0",750,50,FONT_SMALL,CENTER);
      drawText("<ENTER>: Start Game\n<ESC>: Exit Game\n"+
        "\n<CTRL>: Clear Highscores",750,500,FONT_SMALL,CENTER);
      
      //Stop Loop
      //noLoop();
    }
    
    //Draw User Name Question
    else if(_stage == STAGE_NAME)
    { 
      //Draw Logo
      image(LOGO_SMALL,650,100);
      
      //Draw Question
      drawText("Enter your Name: "+_username,750,300,FONT_SMALL,CENTER);
      
      //Start Loop
      loop();
    }
    
    //Draw Level Loader
    else if(_stage == STAGE_LOADER)
    {
      //Draw Logo
      image(LOGO_SMALL,650,100);
      
      //Draw Text
      drawText("Level "+_levelNumber+": "+_level._levelName,750,300,FONT_BIG,CENTER);
      drawText("Press <ENTER> to Start\n\nInstruction\n\n"+
      "Place obstacles to let the leaf fall in the bin."+
      "\nColored obstacles position can be changed, greyscale obstacles are static."+
      "\nUse <MOUSE-LEFT> to move obstacles."+
      "\nPress <SPACE> to let a leaf fall form the Drop Position"
      ,750,400,FONT_SMALL,CENTER);
      
      //Stop Loop
      noLoop();
    }
    
    //Draw Level
    else if(_stage == STAGE_LEVEL)
    {
      //Check for Finish
      if(_level._levelFinished)
      {
        finishLevel();
      }
      else
      {
        //Background
        float addX = map(mouseX,0,width,0,-50);
        float addY = map(mouseY,0,height,0,-50);
        image(BACKGROUND_LEVEL,addX,addY);
      
        //Draw Level
        _level.draw();
    
        //Informations
        drawText("Level "+_levelNumber+": "+_level._levelName,20,30,FONT_SMALL,LEFT);
        drawText("Zeit: "+formatTime(round(millis()-_timeLevelStart)/1000),750,30,FONT_SMALL,CENTER);
        drawText("FPS: "+(int)frameRate,1480,30,FONT_SMALL,RIGHT);
        
        //Start Loop
        loop();
      }
    }
    
    //Draw Level Summary
    else if(_stage == STAGE_END)
    { 
      //Draw Logo
      image(LOGO_SMALL,650,100);
      
      //Calculate & Add Score
      int score = (10000-_timeLevelFinish/100-_level._ballManager._droppings*200);
      HighscoreList hlist = new HighscoreList(this,"data/levels/level_"+_levelNumber+"_scores.xml");
      hlist.addScore(_username,score);
      
      //Read Highscores
      String highscores = hlist.computeTop(5);
      
      //Draw Texts
      drawText("Level "+_levelNumber+": "+_level._levelName,750,300,FONT_BIG,CENTER);
      
      //Draw Text
      String string = "Your Time: "+formatTime(_timeLevelFinish/1000)+"\nNeeded Balls:"
        +_level._ballManager._droppings+"\n\nYour Score: "+score;
      drawText(string,750,400,FONT_SMALL,CENTER);
      
      //Draw Highscores
      drawText(highscores,750,550,FONT_SMALL,CENTER);
      
      //Stop Loop
      noLoop();
    }
  }
  
  //Key Listerner
  void keyPressed()
  {
    //Switch KeyCode
    switch(keyCode)
    {
      //ON Enter Press
      case ENTER:
        if(_stage == STAGE_MENU)
        {
          //Load enter Username Screen
          _stage = STAGE_NAME;
          redraw();
        }
        else if(_stage == STAGE_NAME)
        {
          //Load First Level
          loadLevel(_levelNumber);
          redraw();
        }
        else if(_stage == STAGE_LOADER)
        {
          //Start Loaded Level
          startLevel();
          redraw();
        }
        else if(_stage == STAGE_END)
        {
          //On Game End Load next Level
          loadLevel(_levelNumber+1);
          redraw();
        }
        break;
      
      //On Space Drop Ball
      case 32:
        if(_level != null)
        {
          _level._ballManager.dropBall();
        }
        break;
        
      //Go to Next Level
      case ALT:
        finishLevel();
        break;
        
      //Clear Highscores
      case CONTROL:
        clearHighscores();
        break;
      
      //On Username Frame trigger Backspace
      case BACKSPACE:
        if(_stage == STAGE_NAME && _username.length() > 0)
        {
          _username = _username.substring(0,_username.length()-1);
        }
        break;
      
      //Default Add Key to Username in Name Stage
      default:
        if(_stage == STAGE_NAME)
        {
          _username = _username+key;
        }
        break;
    }
  }
  
  //Load a Level
  void loadLevel(int number)
  {
    if(number <= _levelMax)
    {
      //Get Data
      _levelNumber = number;
      XMLElement xml = new XMLElement(this,"data/levels/level_"+_levelNumber+".xml");
    
       //Initialize Level
      _level = new Level(_levelNumber,xml);
      
      //Load Level Background
      BACKGROUND_LEVEL = loadImage("data/images/"+_level._levelBackground);
      
      //Set Stage
      _stage = STAGE_LOADER;
    }
    else
    {
      //Return to Level 1 & Show Menu
      _levelNumber = 1;
      _stage = STAGE_MENU;
    }
  }
  
  //Start a Level
  void startLevel()
  {
    //Generate
    _level.generate();
    
    //Set Stage
    _stage = STAGE_LEVEL;
    
    //Set Time
    _timeLevelStart = millis();
  }
  
  //Finish a Level
  void finishLevel()
  {
    //Get Time
    _timeLevelFinish = millis()-_timeLevelStart;
    
    //Set Stage
    _stage = STAGE_END; 
  }

  //Clear Highscores
  void clearHighscores()
  {
    //Loop through Levels
    for(int i=1; i <= _levelMax; i++)
    {
      //Get List
      HighscoreList hlist = new HighscoreList(this,"data/levels/level_"+i+"_scores.xml");
      
      //Clear List
      hlist.clear();
    }
  }
  
  //Contact Event Listener
  void contactStarted(FContact c)
  {
    _soundBall.trigger();
  }
  
  //Stopp Override
  void stop()
  {
    //Stop Sounds
    minim.stop();
    
    //Call Super
   super.stop(); 
  }
  
  //Check Mouse
  void mousePressed()
  {
    //Check Level
    if(_level != null)
    {
      _level._ballManager.removeBall(); 
    }
  }

