  
  /*
  *  The Level Controls Physics Elements and delegates Actions
  */
  
  class Level
  {
    //Engine
    FWorld _world;
    
    //Settings
    int _levelNumber;
    String _levelName;
    String _levelBackground;
    
    //XML Data
    XMLElement _staticObstacles;
    XMLElement _useableObstacles;
    
    //Controls
    boolean _mousePressingLeft = false;
    boolean _mousePressingRight = false;
    boolean _levelFinished = false;
    
    //Images
    PImage _imageDrop;
    PImage _imageBin;
    PImage _imageBall;
    
    //Managers
    BallManager _ballManager;
    BinManager _binManager;
    ObstacleManager _obstacleManager;
    
    //Constructor
    Level(int number, XMLElement xml)
    {
      //Set Level Number
      _levelNumber = number;
      
      //Level Settings
      _levelName = xml.getChild(0).getString("name");
      _levelBackground = xml.getChild(0).getString("background");
      
      //World Configuration
      _world = new FWorld();
      _world.setGravity(0,3000);
      
      //initialize BallManager
      _ballManager = new BallManager(xml.getChild(1).getInt("x"),xml.getChild(1).getInt("y"),_world);
      _binManager = new BinManager(xml.getChild(2).getInt("x"),xml.getChild(2).getInt("y"),_world);
      _obstacleManager = new ObstacleManager(_world);
      
      //Get XML Data
      _staticObstacles = xml.getChild(3);
      _useableObstacles = xml.getChild(4);
    }
    
    //Generate Level
    void generate()
    {
      //Set Bin Position
      _binManager.addBin();
      
      //Generate Obstacles
      
      _obstacleManager.process(_useableObstacles,true);
      _obstacleManager.process(_staticObstacles,false);
    }
        
    //Draw Cycle
    void draw()
    {
      //Calculate
      _world.step();
    
      //Draw World
      _world.draw();
      
      //Draw BallManager
      _ballManager.draw();
      
      //Check Wind
      _obstacleManager.checkWind(_ballManager._ball);
      
      //Check Bin
      _levelFinished = _binManager.checkBall(_ballManager._ball);
    }
  }
