  
  /*
  *  The ObstacleManager manages obstacles and save some Settings
  */
  
  class ObstacleManager
  {
    //Objects
    FWorld _world;
    
    //Lists
    ArrayList _rects;
    ArrayList _circles;
    ArrayList _winds;
    
    //Arrays
    int[] _windSpeedsX;
    int[] _windSpeedsY;
    
    //Constructor
    ObstacleManager(FWorld world)
    {
      //Settings
      _world = world;
      
      //Init Arrays
      _rects = new ArrayList();
      _circles = new ArrayList();
      _winds = new ArrayList();
      _windSpeedsX = new int[0];
      _windSpeedsY = new int[0];
    }
    
    //Generate Obstacles
    void process(XMLElement items, boolean useable)
    {
      //Loop through
      for(int i = 0; i < items.getChildCount(); i++)
      {
        //Get Data
        XMLElement item = items.getChild(i);
        String name = item.getName();
      
        //Switch type
        if(name.equals("rect")) addRect(useable,item);
        else if (name.equals("circle")) addCircle(useable,item);
        else if (name.equals("wind")) addWind(useable,item);
      }
    }
  
    //Generate Box
    void addRect(boolean useable, XMLElement item)
    {
      //Get Data
      int x = item.getInt("x");
      int y = item.getInt("y");
      String type = item.getString("type");
      int nrotation = item.getInt("rotation");
      float rotation;
      int nwidth = 200;
      int nheight = 50;
      PImage img = loadImage("data/images/rect_small.png");
      
      //Calc Rotation
      rotation = map(nrotation,0,360,0,TWO_PI);
      
      //Switch type for long & mid -- small is default
      if(type.equals("long"))
      {
        nwidth = 600;
        img = loadImage("data/images/rect_long.png");
      }
      else if(type.equals("mid"))
      {
        nwidth = 400;
        img = loadImage("data/images/rect_mid.png");
      }
      
      //Create Box
      FBox box = new FBox(nwidth,nheight);
      box.setNoStroke();
      box.setFill(0);
      box.setStatic(true);
      box.setPosition(x,y);
      box.setRotation(rotation);
      
      //Useable
      if(useable)
      {
        box.setGrabbable(true);
      }
      else
      {
        box.setGrabbable(false);
        img.filter(GRAY);
        box.setRotatable(false);
      }
  
      //Add to World
      box.attachImage(img);
      _rects.add(box);
      _world.add(box);
    }
  
    //Generate Circle
    void addCircle(boolean useable, XMLElement item)
    {
      //Get Data
      int x = item.getInt("x");
      int y = item.getInt("y");
      
      //Create Circle
      FCircle circle = new FCircle(80);
      circle.setNoStroke();
      circle.setFill(0);
      circle.setStatic(true);
      circle.setPosition(x,y);
      circle.setRotatable(false);
      
      //Load Img
      PImage img = loadImage("data/images/circle.png");
      
      //Useable
      if(useable)
      {
        circle.setGrabbable(true);
      }
      else
      {
        circle.setGrabbable(false);
        img.filter(GRAY);
      }
      
      //Attach Image & Add to World
      circle.attachImage(img);
      _circles.add(circle);
      _world.add(circle);
    }
    
    //Generate Wind
    void addWind(boolean useable, XMLElement item)
    {
      //Get Data
      int x = item.getInt("x");
      int y = item.getInt("y");
      int nwidth = item.getInt("width");
      int nheight = item.getInt("height");
      int speedx = item.getInt("speedx");
      int speedy = item.getInt("speedy");
       
      //Load Img
      PImage img = loadImage("data/images/wind.png");
      img.resize(nwidth,nheight);
      
      //Create Wind
      FBox wind = new FBox(nwidth,nheight);
      wind.setNoStroke();
      wind.setFill(0,127);
      wind.setStatic(true);
      wind.setPosition(x,y);
      wind.setRotatable(false);
      wind.setSensor(true);
      
      //Useable
      if(useable)
      {
        wind.setGrabbable(true);
      }
      else
      {
        wind.setGrabbable(false);
        img.filter(GRAY);
      }
 
       //Attach Image & Add to World
      wind.attachImage(img);
      _winds.add(wind);
      _windSpeedsX = append(_windSpeedsX,speedx);
      _windSpeedsY = append(_windSpeedsY,speedy);
      _world.add(wind);
    }
    
    //Check Wind
    void checkWind(FCircle ball)
    {
      //Loop Wind Objects
      for(int i=0; i < _winds.size(); i++)
      {
        //Get Wind
        FBox wind = (FBox)_winds.get(i);
        
        //Check
        if(wind.isTouchingBody((FBody)ball))
        {
          //Add Force
          ball.addForce(_windSpeedsX[i]*1000,_windSpeedsY[i]*1000);
        }
      }
    }
  }
  
